#!/usr/bin/perl 
#
# WFTrans
# Main Module
#
# (c) 2005 by Peter_Siegrist(SystemLoesungen)  (PSS@ZweierNet.ch)
#
# All Rights reserved.
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
# GNU General Public License for more details.    
#


use strict;
use locale;
use POSIX qw(locale_h);
use POSIX qw(strftime);
use CGI; $| = 1; # flush
use CGI::Carp qw( fatalsToBrowser);
use Text::Balanced qw(extract_tagged);
use File::stat qw(:FIELDS);
use File::Basename;
use File::Path;
use NetAddr::IP;
use IO::Handle;

use lib "./lib";
use WFTrans::Messages;


#--
sub TRUE  { 1; }
sub FALSE { 0; }


#-- Locale Settings
my $LOCALE				= "de_DE.ISO8859-1";


#-- 
#====== USER SETTINGS ================================
# DOCUMENT_ROOT: Document root of the Web-Server (get with `httpd -V`)
my $DOCUMENT_ROOT		= "/opt/httpd/htdocs";

# WEB_LOCATION: subdirectory of DOCUMENT_ROOT where the users directory-links are located.
# This links points to to the users directories in DATA_ROOT.
my $WEB_LOCATION		= "/wftrans_data";

# DATA_ROOT: directory containing the users sub-directories. The data will be stored in this 
# subdirectories. !! This directory must have read/write permission for the web-server !!
my $DATA_ROOT			= "/opt/wftrans";

# LOGFILE: Filename of logfile. This file must be accessible by Webserver.
my $LOGFILE 			= '/var/log/wftrans.log';

# MAX_SIZE_UPLOAD: max. size for uploaded file in KB
my $MAX_SIZE_UPLOAD 	= 50000;	# max upload size in KB

# ALLOW_PWD_FROM: List of CIDR IP-address or Net-address allowed to change passwords
# if the user is MASTER or ADMIN. e.g. (192.168.2.100 10.1.1.1 192.168.1.0/24)
my @ALLOW_PWD_FROM		= qw(192.168.1.0/24);

# UID: Unix NUMERIC! user-id of the webserver process
my $UID					= 65530;	# uid des apache task
# GID: Unix NUMERIC! group-id of the webserver process
my $GID					= 65530;  # gid des apache task

# LOGO: Title text etc. html formatted
my $LOGO				= "<p class=logo2>ZweierNet.ch</p><p class=logo>WFTrans</p>";

# Want Debug Infos ? set it to TRUE
my $DEBUG				= FALSE;
#====== END USER SETTINGS =============================



#-- Vars
my $DATA_ROOT_TMP		= "${DATA_ROOT}/_TMP_HOOK_";
my $USER_DATA_ROOT		= "";
my $USER_LEVEL			= "";
my $USER_PW				= "";
my $LOG					= "";
my $PWDFILE				= "pwdfile";
my $PWDTEMP				= "pwdfile.tmp";
my $CHECKED				= FALSE;
my $MOD_PATH			= FALSE;
my $TMP_AJAX;

my $salt        		= join '', ('.', '/', 0..9, 'A'..'Z', 'a'..'z')[rand 64, rand 64];  #  random two character string

#-- Levels
my $LEVEL;
my %LEV					= ( 0 => 'GUEST_DOWN',
							2 => 'GUEST_UP',
							4 => 'GUEST',
							6 => 'MASTER',
							8 => 'ADMIN' );
							
use constant ADMIN		=> 8;
use constant MASTER		=> 6;
use constant GUEST		=> 4;
use constant GUEST_UP	=> 2;
use constant GUEST_DOWN	=> 0;

my $P					= "";


### Vars ----------------------------------
my $user_dir			= "";
my $user    			= "";
my $password			= "";
my $_error 	    		= "";
my $upload_file			= "";
my $file_name			= "";

my $ERROR				= "<font size=+1 color=yellow><b>" . $_error . "</b></font>";


### Javascript ----------------------------
my $lml =<<"EOF_JS";

var myRequest = false;


function startprogress()
{	
	
	
	
	
	if ( document.startform.file.value == "" ) { return false; }
	if ( document.startform.login.value == "" ) { return false; }
	
	// Formatierung
	if (document.getElementById) {
		document.getElementById('progressMeterText').className = 'progressMeterText';
		document.getElementById('progressMeterText').innerHTML = '0 KB  of  ? KB ';
		//document.getElementById('progressBar').style.display = 'box';
		document.getElementById('progressBar').style.width = '0px';
		if ( navigator.appName == "Microsoft Internet Explorer" ) {
			document.getElementById('progressBarBox').style.display = 'inline';
		} else {
			document.getElementById('progressBarBox').style.display = 'table';
		}
		
		document.getElementById('progressText').style.color = 'black';
		document.getElementById('progressText').style.textAlign = 'center';
		document.getElementById('progressText').innerHTML = '                               0%';
		
		
	} else if (document.all) {
		document.all['progressMeterText'].className = 'progressMeterText';
		document.all['progressMeterText'].innerHTML = '0 KB  of  ? KB ';
		//document.all['progressBar'].style.display = 'box';
		document.all['progressBarBox'].style.display = 'table';
		
		document.all['progressText'].style.color = 'black';
		document.all['progressText'].innerHTML = '                               0%';
		
		
	} else if (document.layers) {
		document.layers['progressMeterText'].className = 'progressMeterText';
		document.layers['progressMeterText'].innerHTML = '0 KB  of  ? KB ';
		//document.layers['progressBar'].style.display = 'box';
		document.layers['progressBarBox'].style.display = 'table';
		
		document.layers['progressText'].style.color = 'black';
		document.layers['progressText'].innerHTML = '                               0%';
		
	} else { return false; }
				
		
	var file = document.startform.file.value;
	var user = document.startform.login.value;
	var ufile = user + '/' + file;
	pageReq = "$ENV{SCRIPT_NAME}?action=get_upload_progress&file=" + ufile;
	
	window.setTimeout("handleAJAX(pageReq)", 1200);
	return true;
}


function updateProgress()
{	
	pageReq = document.startform.savetifor.value;
	
	if(myRequest.readyState == 4) {
		if(myRequest.status == 200 || myRequest.status == 0) {
			
			var response = new Array();
			response = myRequest.responseText.split('|');
			// alert(response);
			// format of response = "total_length|processed_bytes" 
			
			var total_length = response[0];
			var processed_bytes = response[1];
			
			if ( total_length == "UNDEF" ) {
				// reactivate handle
				window.setTimeout("handleAJAX(pageReq)", 1200);
				return true;
			}
			
			var processed_percent = Math.ceil((processed_bytes/total_length)*100);
			
			
			// Formatierung
			if (document.getElementById) {
				document.getElementById('progressMeterText').className = 'progressMeterText';
				document.getElementById('progressMeterText').innerHTML = parseInt(processed_bytes/1024) + ' KB  of  ' + parseInt(total_length/1024) + ' KB ';
				document.getElementById('progressBar').style.width = parseInt(processed_percent*3) + 'px';
				
				document.getElementById('progressBar').style.color = '#5F9EA0';
				document.getElementById('progressBar').style.background = '#5F9EA0';
				
				document.getElementById('progressText').className = 'progressText';
				document.getElementById('progressText').style.color = 'black';
				document.getElementById('progressText').style.textAlign = 'center';
				document.getElementById('progressText').innerHTML = '                               ' + processed_percent + '%';
				
				document.getElementById('uploadbutton').disabled = true;
				document.getElementById('downloadbutton').disabled = true;
				
				if ( processed_percent >= "99" ) { 
					document.getElementById('uploadbutton').disabled = false;
					document.getElementById('downloadbutton').disabled = false;
				}
			} else if (document.all) {
				document.all['progressMeterText'].className = 'progressMeterText';
				document.all['progressMeterText'].innerHTML = parseInt(processed_bytes/1024) + ' KB  of  ' + parseInt(total_length/1024) + ' KB ';
				document.all['progressBar'].style.width = parseInt(processed_percent*3) + 'px';
				
				document.all['progressBar'].style.color = '#5F9EA0';
				document.all['progressBar'].style.background = '#5F9EA0';
				
				document.all['progressText'].style.color = 'black';
				document.all['progressText'].style.textAlign = 'center';
				document.all['progressText'].innerHTML = '                               ' + processed_percent + '%';
				
				document.all['uploadbutton'].disabled = true;
				document.all['downloadbutton'].disabled = true;
				
				if ( processed_percent >= "99" ) { 
					document.all['uploadbutton'].disabled = false;
					document.all['downloadbutton'].disabled = false;
				}
			} else if (document.layers) {
				document.all['progressMeterText'].className = 'progressMeterText';
				document.all['progressMeterText'].innerHTML = parseInt(processed_bytes/1024) + ' KB  of  ' + parseInt(total_length/1024) + ' KB ';
				document.all['progressBar'].style.width = parseInt(processed_percent*3) + 'px';
				
				document.all['progressBar'].style.color = '#5F9EA0';
				document.all['progressBar'].style.background = '#5F9EA0';
				
				document.all['progressText'].style.color = 'black';
				document.all['progressText'].style.textAlign = 'center';
				document.all['progressText'].innerHTML = '                               ' + processed_percent + '%';
				
				document.all['uploadbutton'].disabled = true;
				document.all['downloadbutton'].disabled = true;
				if ( processed_percent >= "99" ) { 
					document.all['uploadbutton'].disabled = false;
					document.all['downloadbutton'].disabled = false;
				}
			} else { return; }
			
			
			// reactivate handle
			window.setTimeout("handleAJAX(pageReq)", 1200);
			
		}
		else
		{
			alert('Error: Status-Code: ' + myRequest.status);
		}
	}

}	


function handleAJAX(pageReq)
{
	myRequest = false;

	if(window.XMLHttpRequest) {
		myRequest = new XMLHttpRequest();
		if(myRequest.overrideMimeType) {
			myRequest.overrideMimeType('text/html');
		}
	} else if(window.ActiveXObject) {
		try {
			myRequest = new ActiveXObject("Msxml2.XMLHTTP");
		} catch (e) {
			try {
				myRequest = new ActiveXObject("Microsoft.XMLHTTP");
			} catch (e) {}
		}
	}
	if(!myRequest) {
		alert('Error: Browser versteht keine XMLHttpRequests.');
		return false;
	}
	document.startform.savetifor.value = pageReq;

	myRequest.onreadystatechange = updateProgress;
	myRequest.open('GET', pageReq, true);
	myRequest.send(null);
	
	
	
}

function new_window( url, titel ) {
var w = window.open(url, titel, "toolbar=no, location=no,directories=no,status=no,width=300,height=260,alwaysRaised=yes,scrollbars=no");
}

function reset_button() {
	document.getElementById('uploadbutton').disabled = false;
	document.getElementById('downloadbutton').disabled = false;
}

EOF_JS

#---------------------------------------------------- end js 




#-- Locale Handling
setlocale(LC_CTYPE, "$LOCALE");

					
### Main ----------------------------------

#-- open MyPort logfile

$LOG = &open_log_file( ">>$LOGFILE" );
print $LOG "\n--------- ", scalar localtime, " ------------------------\n";

my $oldfh = select;
select $LOG; $| = 1;
select $oldfh; $| = 1;

$CGI::POST_MAX = 1024 * $MAX_SIZE_UPLOAD;
$CGI::DISABLE_UPLOADS = 0;

my $data;
my $Q = CGI->new(\&hook, $data);
my $SCRIPT_NAME = $Q->script_name();


#-- post error?
if ( $Q->cgi_error() ) {
	print $LOG "CGI-ERROR: ", $Q->cgi_error(), "\n";
	print $Q->header();
	print $Q->start_html( -title => "Error",
			-author => "$AUTHORMAIL",
			-alink => 'red',
			-link => 'ivory',
			-vlink => 'ivory',
			-style => {-code => "$STYLE_1"},
			-BGCOLOR => "$BGCOLOR" );
	print "Upload ERROR: ", $Q->cgi_error();
	print $Q->end_html;
	exit;
}


	
### Delivery  -----------------------------------------

print $LOG $Q->request_method(), " - mit Params: " , $Q->param(), "\n";
if ( $Q->request_method() eq "POST" ) {

	if ( $Q->param('start_upload') ) {			# POST Request
		&upload_datei;
	} elsif ( $Q->param('browse') ) {
		&browse;
	} elsif ( $Q->param('a_start') ) {
		&admin_screen;
	} elsif ( $Q->param('ch_pw') ) {
		&ch_pw;
	} elsif ( $Q->param('del_dat') ) {
		&del_dat;
	} elsif ( map { /^delete_file_*/ } $Q->param() ) {
		&delete_file;
	} elsif ( $Q->param('adm_edit') ) {
		&admin_edit;
	} elsif ( $Q->param('adm_add') ) {
		&admin_add;
	} elsif ( $Q->param('adm_modifyit') ) {
		&admin_modifyit;
	} elsif ( $Q->param('adm_addit') ) {
		&admin_addit;
	} elsif ( $Q->param('adm_delete') ) {
		&adm_delete;
	} else {
		&welcome_screen;
	}
} else {	# GET Method
	if ( $Q->param('action') =~ /get_upload_progress/i ) {
		&get_upload_progress;
		exit;
	}
	if ( $Q->param('mod_pwd') ) {
		&mod_pwd;
		exit;
	} elsif ( $Q->param('admin') ) {
		&admin_start;
		exit;
	} 
	&welcome_screen;
}

exit;


### End of Script ------------------------

end_script:

print $Q->end_html; 

exit;




### Subs ---------------------------------

sub welcome_screen {
	print $LOG "=== welcome_screen\n";
	print $LOG $Q->param(), "\n";
	
	#-- Access Check
	my $APW = FALSE;
	my $from_adr = NetAddr::IP->new($ENV{'REMOTE_ADDR'});
	foreach my $allow_adr ( @ALLOW_PWD_FROM ) {
		my $space = NetAddr::IP->new($allow_adr);
		if ( $space->contains(NetAddr::IP->new($from_adr) ) ) {
			$APW = TRUE;
		}
	}
	
	
	# clean up
	#$Q->delete_all() if not $P;

	my $title = "WFTrans";

	print $Q->header(	-cache_control => 'no-cache',
				-pragma => 'no-cache',
				-expires => 'now',
				-type => 'text/html' );
	
	print $Q->start_html(	-title => "$title",
				-meta => { 	cache_control => 'no-cache',
							expires => 'now',
							pragma => 'no-cache',
							robots => 'noindex' },
				-author => "$AUTHORMAIL",
				-alink => 'red',
				-link => 'ivory',
				-vlink => 'ivory',
				-style => {-code => "$STYLE_1"},
				-script => "$lml",
				-onLoad => "reset_button()",
				-BGCOLOR => "$BGCOLOR" );

	print qq(<center><p><br>$LOGO</center>\n);
	
	print $Q->start_multipart_form(	-method => 'POST', -name => 'startform', -onsubmit=> 'return startprogress()', -action=> "$ENV{SCRIPT_NAME}");
	
	#-- Uploads
	print qq(<hr size=1>);
	print qq(<u><font size=+1>Uploads</font></u><p>);
	print qq(Dateien auf dem Server ablegen.<br>);
	
	print qq(<table>);
	print qq(<tr><td align="right"> Login (Gruppenname) : </td><td><input type="text" name="login"  size=10></td></tr>);
	print qq(<tr><td align="right"> Passwort : </td><td><input type="password" name="passwort"  size=10>);
	$APW ? print qq(&nbsp&nbsp&nbsp&nbsp&nbsp<a href="javascript:new_window('${SCRIPT_NAME}?mod_pwd=1', 'Modify Password')"><u>ndern</u></a></td></tr>) : print qq(</td></tr>);
	print qq(<tr><br><td align="right"> Upload File : </td><td><input type="file" name="file"  size=50></td></tr>);
	print qq(<tr><br><td align="right"> </td><td align="left">);
	print $Q->hidden( -name=>"savetifor", -value=>"");
	print $Q->hidden( -name=>"tmp_ajax_fh", -value=>"");
	print $Q->submit(	-name => "start_upload", -value => "Start Upload", -id=>"uploadbutton");
	print qq(</td></tr>);
	print qq(</table>);

	print "<center><font color=red><b>" . $_error . "</b></font></center>" if $P eq "upload";

	print $Q->endform();
	
	
	##print qq(<center><table id="progressBarBox" class="progressBarBox" style="display: none">);
	##print qq(<tr><td><b>Status: </b></td><td width="300"><div id="progressMeterText"><br></div></td></tr>);
	##print qq(<tr><td><b>Progress: </b></td><td width="300" class="progressBar">);
	##print qq(<div id="progressBar" class="progressBar"><div id="progressText" class="progressText"></div></div></td></tr></table>);
	##print qq(</center>);
	
	print qq(<center><table id="progressBarBox" class="progressBarBox" style="display: none">);
	print qq(<tr><td><b>Status: </b></td><td width="300"><div id="progressMeterText"><br></div></td></tr>);
	print qq(<tr><td><b>Progress: </b></td><td width="300" class="progressBar">);
	print qq(<div id="progressBar" class="progressBar"><div id="progressText" align="center" class="progressText"></div></div></td></tr></table>);
	print qq(</center>);
	
	
		
	#-- Dowloads
	print $Q->start_multipart_form(	-method => 'POST', -name => 'startform2');
	
	print qq(<p><br><hr size=1>);
	print qq(<u><font size=+1>Downloads</font></u><p>);
	print qq(Dateien vom Server abholen / Dateien lschen.<br>);
	
	print qq(<table>);
	print qq(<tr><td align="right"> Login (Gruppenname) : </td><td><input type="text" name="login"  size=10></td></tr>);
	print qq(<tr><td align="right"> Passwort : </td><td><input type="password" name="passwort"  size=10>);
	$APW ? print qq(&nbsp&nbsp&nbsp&nbsp&nbsp<a href="javascript:new_window('${SCRIPT_NAME}?mod_pwd=1', 'Modify Password')"><u>ndern</u></a></td></tr>) : print qq(</td></tr>);
	print qq(<tr><br><td align="right"> </td><td align="left">);
	print $Q->submit(	-name => "browse", -value => "Verzeichnis anzeigen", -id=>"downloadbutton");
	print qq(</td></tr>);
	print qq(</table>);
	
	print "<center><font color=red><b>" . $_error . "</b></font></center>" if $P eq "browse";

	print $Q->endform();
	
	
	print $FOOTER_MAIN_A;
	
	goto end_script;
	
}



sub browse {
	print $LOG "=== browse\n";
	print $LOG $Q->param(), "\n";
	$P = "browse";
	
	$user = $Q->param('login');
	$password = $Q->param('passwort');
	
	#-- User Check
	if ( ! &check_user($user, $password) ) {
		&welcome_screen;
	}
	
	#-- Check rights
	if ( $USER_LEVEL == GUEST_UP ) {
		$_error = "No authorization.";
		&welcome_screen;
		exit;
	}
	
	my $title = "Browse " . basename($user_dir);
	
	print $Q->header(	-cache_control => 'no-cache',
				-pragma => 'no-cache',
				-expires => 'now',
				-type => 'text/html' );
	
	print $Q->start_html(	-title => "$title",
				-meta => { 	cache_control => 'no-cache',
							expires => 'now',
							pragma => 'no-cache',
							robots => 'noindex' },
				-author => "$AUTHORMAIL",
				-alink => 'red',
				-link => 'ivory',
				-vlink => 'ivory',
				-style => {-code => "$STYLE_1"},
				-BGCOLOR => "$BGCOLOR" );

	print qq(<center><p><br>$LOGO</center>\n);
	
	print $Q->start_multipart_form(	-method => 'POST', -name => 'startform');
	
	my @filelist = <${user_dir}/*>;
	
	print qq(<hr size=1><br><b><u>Dateien zum download fr das Gruppenverzeichnis <i> $user </i>: </u></b><p>);
	
	print qq(<table border=1 width=80%><th align=left>Filename <font size=-2>(Click for download)</font></th><th align=left>Datum </th><th align=left>Grsse in Byte</th><th> </th>);
	foreach my $line ( @filelist ) {
		next if ! -f $line;				# keine reale datei
		stat($line);
		my $fn = basename($line);
		my $ft = strftime("%d.%m.%Y %H:%M", localtime($st_ctime));
		my $fs = $st_size;
		if ( $USER_LEVEL <= GUEST ) {
			print qq(<tr bgcolor=$LISTBG><td width=60% nowrap><a href="$WEB_LOCATION/$user/$fn">$fn</a></td><td nowrap>$ft</td><td nowrap>$fs</td></tr>);
		} elsif ( $USER_LEVEL >= MASTER ) {
			print qq(<tr bgcolor=$LISTBG><td width=60% nowrap><a href="$WEB_LOCATION/$user/$fn">$fn</a></td><td nowrap>$ft</td><td nowrap>$fs</td>);
			print qq(<td nowrap align="center">);
			print $Q->submit(	-name => "delete_file_$WEB_LOCATION/$user/$fn", -value => " lschen ", -class=>"delbut");
			print qq(</td></tr>);
		}
	}
	print qq(</table>);
	
	print $Q->hidden( "login", "$user" );
	print $Q->hidden( "passwort", "$password" );
	
	print $FOOTER;
	
	print $Q->endform();
	
	goto end_script;
}


sub upload_datei {
	print $LOG "=== upload\n";
	print $LOG $Q->param(), "\n";
	$P = "upload";
	
	
	$user = $Q->param('login');
	$password = $Q->param('passwort');
	$file_name = $Q->param('file');
	#-- extract filename
	$file_name =~ s/\w\://;		# Driveletter (iexplodierer)
	$file_name =~ s/\\\w+\\//;	# win-path (iexplodierer)


	
	#-- User Check
	if ( ! &check_user($user, $password) ) {
		&welcome_screen;
	}

	#-- Check rights
	if ( $USER_LEVEL < GUEST_UP ) {
		$_error = "No authorization.";
		&welcome_screen;
		exit;
	}

	
	if (! $file_name) {
		$_error = "Bad Filename !";
		&welcome_screen;
	}

	
	if (-e "$user_dir/$file_name") {
		$_error = "Filename already exists, choose an other one !";
		&welcome_screen;
	}
	
	
	#-- Upload
    
    
    if ( ! open(FILE,">$user_dir/$file_name") ) {
    	$_error = "Error opening file $file_name for writing: $!";
    	&welcome_screen;
    }
    
    my $size = my $bc = 0;
    binmode FILE;


	$upload_file = $Q->upload('file');
    my $buff;
    while ($bc = read( $upload_file, $buff, 2048)) {
    	
    	$size += $bc;
        print FILE $buff;
    }
    close(FILE);

	
    

	
	my $title = "Upload " . basename($file_name);
	
	print $Q->header(	-cache_control => 'no-cache',
				-pragma => 'no-cache',
				-expires => 'now',
				-type => 'text/html' );
	
	print $Q->start_html(	-title => "$title",
				-meta => { 	cache_control => 'no-cache',
							expires => 'now',
							pragma => 'no-cache',
							robots => 'noindex' },
				-author => "$AUTHORMAIL",
				-alink => 'red',
				-link => 'ivory',
				-vlink => 'ivory',
				-style => {-code => "$STYLE_1"},
				-BGCOLOR => "$BGCOLOR" );

	print qq(<center><p><br>$LOGO</center>\n);
	
	print qq(<hr size=1><br>);
	
	
	#-- No Space ?
	stat( "$user_dir/$file_name" );
	if ( $st_size <= 0 ) {
		print "<b><font color=red>No space left on disk. Operation aborted !</font></b>";
		unlink "$user_dir/$file_name";
		print $FOOTER;
		goto end_script;
	}
	
	chmod (0660, "$user_dir/$file_name");
	
	unlink "${DATA_ROOT_TMP}/${file_name}_TMP_AJAX" or print $LOG "ERR_TMP_AJAX_DELETE: ${DATA_ROOT_TMP}/${file_name}_TMP_AJAX err: $!\n";
	
	
    
	if ( $USER_LEVEL == GUEST_UP ) {
		#-- ok , succeeded	
		print "<p><br><center><font size=+1>Upload of <i>$file_name ($size Bytes)</i> succeeded.</font></center>";
		print $Q->hidden( "login", "$user" );
		print $Q->hidden( "passwort", "$password" );	
		print $FOOTER;
		print $Q->endform();
	
		goto end_script;
	}
	
	#-- ok , succeeded	
	print "<center><font size=+1>Upload of <i>$file_name ($size Bytes)</i> succeeded.</font></center>";
	
	print $Q->start_multipart_form(	-method => 'POST', -name => 'startform');
	
	my @filelist = <${user_dir}/*>;
	
	print qq(<p><br><b><u>Dateien im Gruppenverzeichnis <i> $user </i>: </u></b><p>);
	
	print qq(<table border=1 width=80%><th align=left>Filename <font size=-2>(Click for download)</font></th><th align=left>Datum </th><th align=left>Grsse in Byte</th><th> </th>);
	foreach my $line ( @filelist ) {
		next if ! -f $line;				# keine reale datei
		stat($line);
		my $fn = basename($line);
		my $ft = strftime("%d.%m.%Y %H:%M", localtime($st_ctime));
		my $fs = $st_size;
		if ( $USER_LEVEL <= GUEST ) {
			print qq(<tr bgcolor=$LISTBG><td width=60% nowrap><a href="$WEB_LOCATION/$user/$fn">$fn</a></td><td nowrap>$ft</td><td nowrap>$fs</td></tr>);
		} elsif ( $USER_LEVEL >= MASTER ) {
			print qq(<tr bgcolor=$LISTBG><td width=60% nowrap><a href="$WEB_LOCATION/$user/$fn">$fn</a></td><td nowrap>$ft</td><td nowrap>$fs</td>);
			print qq(<td nowrap align="center">);
			print $Q->submit(	-name => "delete_file_$WEB_LOCATION/$user/$fn", -value => " lschen ", -class=>"delbut");
			print qq(</td></tr>);
		}
	}
	print qq(</table>);
	
	print $Q->hidden( "login", "$user" );
	print $Q->hidden( "passwort", "$password" );
	
	print $FOOTER;
	
	print $Q->endform();
	
	goto end_script;
}


sub hook {
	print $LOG "HOOK\n" if $DEBUG;
	my ($filename, $buffer, $bytes_read, $data) = @_;
	my $upload_file_size = $ENV{CONTENT_LENGTH};

print $LOG "> $filename: Filesize:$upload_file_size, Bytes_Read: $bytes_read\n" if $DEBUG;

	#-- extract filename
	$filename =~ s/\w\://;		# Driveletter ( danke IE)
	$filename =~ s/\\\w+\\//;	# win-path (danke IE)
	
	#print "Cache-Control: no-store, no-cache\n";
	#print "Content-type: text/html\n\n";
	#
	#$bytes_read ? print "$upload_file_size|$bytes_read" : print "UNDEF|UNDEF";
	
	
	#my $Q = new CGI; 
	
	
	###==============================
	##
	##$user = $Q->param('login');
	##$password = $Q->param('passwort');
	##print $LOG "> $user / $password\n";
	##
	###-- User Check
	##if ( ! &check_user($user, $password) ) {
	##	&welcome_screen;
	##}
    ##
	###-- Check rights
	##if ( $USER_LEVEL < GUEST_UP ) {
	##	$_error = "No authorization.";
	##	&welcome_screen;
	##	exit;
	##}
    ##
	##
	##if (! $filename) {
	##	$_error = "Bad Filename !";
	##	&welcome_screen;
	##}
    ##
	##
	##if (-e "$user_dir/$filename") {
	##	$_error = "Filename already exists, choose an other one !";
	##	&welcome_screen;
	##}
	##
	##
	##
	### ========================================
	##
	
	
	
	
    if ( ! open(TMP_AJAX,">${DATA_ROOT_TMP}/${filename}_TMP_AJAX") ) {
    	$_error = "Error opening file ${DATA_ROOT_TMP}/${filename}_TMP_AJAX for writing: $!";
    	print $LOG "Error opening file ${DATA_ROOT_TMP}/${filename}_TMP_AJAX for writing: $!\n";
    	&welcome_screen;
    }
	seek TMP_AJAX, 0, 0;
	print TMP_AJAX "$upload_file_size|$bytes_read\n";
    close TMP_AJAX;
    
}


sub del_dat {
	print $LOG "=== del_dat\n";
	print $LOG $Q->param(), "\n";
	$P = "browse";
	
	$user = $Q->param('login');
	$password = $Q->param('passwort');
	
	#-- User Check
	if ( ! &check_user($user, $password) ) {
		&welcome_screen;
	}
	
	#-- Check rights
	if ( $USER_LEVEL <= GUEST ) {
		$_error = "No authorization.";
		&welcome_screen;
		exit;
	}
	
	my $title = "Remove Files " . basename($user_dir);
	
	print $Q->header(	-cache_control => 'no-cache',
				-pragma => 'no-cache',
				-expires => 'now',
				-type => 'text/html' );
	
	print $Q->start_html(	-title => "$title",
				-meta => { 	cache_control => 'no-cache',
							expires => 'now',
							pragma => 'no-cache',
							robots => 'noindex' },
				-author => "$AUTHORMAIL",
				-alink => 'red',
				-link => 'ivory',
				-vlink => 'ivory',
				-style => {-code => "$STYLE_1"},
				-BGCOLOR => "$BGCOLOR" );

	print qq(<center><p><br>$LOGO</center>\n);
	
	print $Q->start_multipart_form(	-method => 'POST', -name => 'startform');
	
	my @filelist = <${user_dir}/*>;
	
	print qq(<hr size=1><br><b><u>Dateien im Gruppenverzeichnis <i> $user </i>: </u></b><p>);
	
	print qq(<table border=1 width=80%><th align=left>Filename <font size=-2> (Click for download)</font></th><th align=left>Datum </th><th align=left>Grsse in Byte</th><th> </th>);
	foreach my $line ( @filelist ) {
		next if ! -f $line;				# keine reale datei
		stat($line);
		my $fn = basename($line);
		my $ft = strftime("%d.%m.%Y %H:%M", localtime($st_ctime));
		my $fs = $st_size;
		print qq(<tr bgcolor=$LISTBG><td width=60% nowrap><a href="$WEB_LOCATION/$user/$fn">$fn</a></td><td nowrap>$ft</td><td nowrap>$fs</td>);
		print qq(<td nowrap>);
		print $Q->submit(	-name => "delete_file_$WEB_LOCATION/$user/$fn", -value => " lschen ");
		print qq(</td></tr>);
	}
	print qq(</table>);
	
	print $Q->hidden( "login", "$user" );
	print $Q->hidden( "passwort", "$password" );
	
	print $FOOTER;
	
	print $Q->endform();
	
	goto end_script;
}


sub delete_file {
	print $LOG "=== delete_file\n";
	print $LOG $Q->param(), "\n";

	my $dfname;
	
	$user = $Q->param('login');
	$password = $Q->param('passwort');
	
	#-- User Check
	if ( ! &check_user($user, $password) ) {
		&welcome_screen;
	}
	
	my @p = $Q->param();
	
	foreach my $line ( @p ) { 
		if ( $line =~ /^delete_file_/ ) {
			$line =~ s/delete_file_//;
			$dfname = $line;
			
		}
	}
	chomp($dfname);
	$dfname = basename($dfname);
	
	if ( $dfname ne "" ) {
		unlink "$user_dir/$dfname";
		print $LOG "DELETE FILE: $user_dir/$dfname\n";
	}
	
	#&del_dat;
	&browse;
	exit;
	
}


sub check_user( $$ ) {
    my $user        = shift;
    my $pass        = shift;

    my $login       = FALSE;

    if ( $user && $pass ) {
        if ( !open(PWD, "$PWDFILE") ) {
            $_error = "Cannot open $PWDFILE: $!";
            return FALSE;
        }
        my @pwd = <PWD>;
        close(PWD);

		$_error = "No valid User/Group found !";
        foreach ( @pwd ) {
			chomp;
            if ( /^$user\|(.*)$/ ) {    # get the password
            	$_error = "";
            	my $rest = $1;
            	my ($a, $b, $c) = split('\|', $rest);
                my $crypted = $a;
                $USER_LEVEL = $b;
                $user_dir = $c;
                if ( ! -d $user_dir ) {
                	$_error = "Directory not found !";
                	return FALSE;
                }
                if ( crypt($pass, $crypted) eq $crypted ) { 
                    $login = TRUE; $CHECKED = TRUE;
                    last;
                } else {
                	$_error = "User/Password dont match !";
                	last;
                }
            }
        }
    } else {
    	$_error = "Login & Password required.";
    	return FALSE;
    }
    
    return $login ? TRUE : FALSE;
}



sub read_user_data ( $ ) {
	my $user 		= shift;
    my $ok	        = FALSE;

    if ( $user ) {
        my @pwd = &read_data_file;

		$_error = "No valid User/Group found !";
        foreach ( @pwd ) {
			chomp;
            if ( /^$user\|(.*)$/ ) {    # get the password
            	$ok = TRUE;
            	$_error = "";
            	my $rest = $1;
            	my ($pw, $level, $data_root, $web_location) = split('\|', $rest);
            	
            	$USER_PW = $pw;
            	$WEB_LOCATION = $web_location;
            	$USER_DATA_ROOT = $data_root;
            	$LEVEL = $level;
            }
        }
    } else {
    	$_error = "No User.";
    	return FALSE;
    }
    
    return $ok ? TRUE : FALSE;
}


sub read_data_file () {
    #-- Struktur pwdfile:
    # user|password|level|data_root|web_location
    
    if ( !open(PWD, "$PWDFILE") ) {
        $_error = "Cannot open $PWDFILE: $!";
        return FALSE;
    }
    
    my @data = <PWD>;
    close(PWD);

    return @data;
}


sub write_data  {
	print $LOG "=== write_data\n";
	print $LOG $Q->param(), "\n";

	my $login 			= shift;
	my $passwort 		= shift;
	my $level 			= shift;
	my $user_data_path 	= shift;
	my $web_location 	= shift;
	
	
	if ( $MOD_PATH ) {
		rmtree( $USER_DATA_ROOT ) if noslash($USER_DATA_ROOT) ne noslash($DATA_ROOT);
		rmtree( "$DOCUMENT_ROOT/$web_location/$login" ) if noslash("$DOCUMENT_ROOT/$WEB_LOCATION/$login") ne noslash("$DOCUMENT_ROOT/$WEB_LOCATION/");
	}
		
		
	mkdir $user_data_path, 0750;
	chown ($UID, $GID, $user_data_path);
	$_error =  "Cannot create directory $user_data_path : $!\n" if ! -d "$user_data_path";
	return FALSE if ! -d "$user_data_path";
	
	symlink $user_data_path, "$DOCUMENT_ROOT/$web_location/$login";
	$_error =  "Cannot create symbolic link $DOCUMENT_ROOT/$web_location/$login : $!\n" if ! -l "$DOCUMENT_ROOT/$web_location/$login";
	return FALSE if ! -l "$DOCUMENT_ROOT/$web_location/$login";
		
    if ( ! open(PWD_OUT, ">$PWDTEMP") ) {
    	$_error = "Cannot open $PWDTEMP: $!";
    	return FALSE;
    }		
	if ( ! open(PWD, "$PWDFILE") ) {
		$_error = "Cannot open $PWDFILE: $!";	
		return FALSE;
	}

	my $found = FALSE;
	while ( my $line = <PWD> ) {
	    if ( $line =~ /^$login\|/ ) {
	        $line = "$login\|" . crypt($passwort, $salt) . "\|$level" . "\|$user_data_path" . "\|$web_location" . "\n";
	        $found = TRUE;
	    }
	    print PWD_OUT $line;
	}
	close PWD;
	
	print PWD_OUT "$login\|" . crypt($passwort, $salt) . "\|$level" . "\|$user_data_path" . "\|$web_location" . "\n" unless $found;
	
	close PWD_OUT;

	system("mv $PWDTEMP $PWDFILE");
	
	return TRUE;
}


sub write_pw  {
	print $LOG "=== write_data\n";
	print $LOG $Q->param(), "\n";

	my $login 			= shift;
	my $passwort 		= shift;
		
    if ( ! open(PWD_OUT, ">$PWDTEMP") ) {
    	$_error = "Cannot open $PWDTEMP: $!";
    	return FALSE;
    }		
	if ( ! open(PWD, "$PWDFILE") ) {
		$_error = "Cannot open $PWDFILE: $!";	
		return FALSE;
	}

	my $found = FALSE;
	while ( my $line = <PWD> ) {
	    if ( $line =~ /^$login\|(.*)$/ ) {
	    	my $rest = $1;
            my ($pw, $level, $data_root, $web_location) = split('\|', $rest);
	        $line = "$login\|" . crypt($passwort, $salt) . "\|$level" . "\|$data_root" . "\|$web_location" . "\n";
	        $found = TRUE;
	    }
	    print PWD_OUT $line;
	}
	close PWD;
	close PWD_OUT;

	system("mv $PWDTEMP $PWDFILE");
	
	return $found ? TRUE : FALSE;
}


sub open_log_file {
	open( my $fh, "@_" ) or die "Cannot open Logfile \"@_\" : $!\n";
	return $fh;
}


sub noslash ( $ ) {
	my $path = shift;	

	$path =~ s/\///g;	
	return $path;
}

	
sub mod_pwd {
	print $LOG "=== mod_pwd\n";
	print $LOG $Q->param(), "\n";
	print $LOG "Chng_pwd from: $ENV{'REMOTE_ADDR'}\n";
	
	my $from_adr = NetAddr::IP->new($ENV{'REMOTE_ADDR'});
	foreach my $allow_adr ( @ALLOW_PWD_FROM ) {
		my $space = NetAddr::IP->new($allow_adr);
		if ( ! $space->contains(NetAddr::IP->new($from_adr) ) ){
			&welcome_screen;
		}
	}
		
	print $Q->header();
	print $Q->start_html( -title => "Modify Password",
			-meta => { 	cache_control => 'no-cache',
							expires => 'now',
							pragma => 'no-cache',
							robots => 'noindex' },
			-author => "$AUTHORMAIL",
			-alink => 'red',
			-link => 'ivory',
			-vlink => 'ivory',
			-style => {-code => "$STYLE_1"},
			-BGCOLOR => "$BGCOLOR" );
			
	print qq(<center><p class=mid>Gruppenpasswort ndern</p>);
	
	print qq(<table>);
	print $Q->start_multipart_form(	-method => 'POST', -name => 'pwd_form');
	print qq(<tr><td align="right"> Master Login : </td><td><input type="text" name="ma_login"  size=10></td></tr>);
	print qq(<tr><td align="right"> Master Passwort : </td><td><input type="password" name="ma_passwort"  size=10></td></tr>);
	print qq(<tr><td align="left"><p> </p><u>Passwort ndern fr Gruppe :</u></td><td></td></tr>);
	print qq(<tr><td align="right"> Login (Gruppenname) : </td><td><input type="text" name="login"  size=10></td></tr>);
	print qq(<tr><td align="right"> Neues Passwort : </td><td><input type="password" name="passwort_new"  size=10></td></tr>);
	print qq(<tr><br><td align="right"> </td><td>);
	print $Q->submit(	-name => "ch_pw", -value => "ndern");
	print qq(</td></tr>);
	print qq(</table>);
	print $Q->endform();
	print qq(<hr size=1></center>);
	
	goto end_script;
}


sub ch_pw {
	print $LOG "=== ch_pw\n";
	print $LOG $Q->param(), "\n";

	my $login 			= $Q->param('login');
	my $passwort_new 	= $Q->param('passwort_new');
	
	
	print $Q->header();
	print $Q->start_html( -title => "Modify Password",
			-author => "$AUTHORMAIL",
			-alink => 'red',
			-link => 'ivory',
			-vlink => 'ivory',
			-style => {-code => "$STYLE_1"},
			-BGCOLOR => "$BGCOLOR" );

	print qq(<center><p class=mid>Gruppenpasswort ndern</p>);
	
	#-- check master 
	if ( $passwort_new eq "" ) {
		print qq(<p class=mid color=blue>Leeres Passwort nicht erlaubt.</p>);
		print qq(<a href="javascript:window.close()"> close </a>);
		exit;
	}
	if ( &check_user( $Q->param('ma_login'), $Q->param('ma_passwort') ) ) {
		#-- check level
		if ( $USER_LEVEL >= MASTER ) {
			#-- check user
			if ( &read_user_data( $login ) ) {
						
				if ( &write_pw ( $login, $passwort_new, $LEVEL, $USER_DATA_ROOT, $WEB_LOCATION ) ) {
					print qq(<p class=mid color=blue>Passwort gendert</p>);
					print qq(<a href="javascript:window.close()"> close </a>);
					exit;
				} else {
					print qq(<p class=mid color=yellow>ERR: $_error</p>);
					print qq(<a href="javascript:window.close()"> close </a>);
					exit;
				}
			}  else {
				print qq(<p class=mid color=yellow>Ungltige Gruppe.</p>);
				print qq(<a href="javascript:window.close()"> close </a>);
				exit;
			}
		} else {
				print qq(<p class=mid color=yellow>Keine Berechtigung (L).</p>);
				print qq(<a href="javascript:window.close()"> close </a>);
				exit;
		}
	} else {
				print qq(<p class=mid color=yellow>Nicht Master.</p>);
				print qq(<a href="javascript:window.close()"> close </a>);
				exit;
	}
	
	
	exit;
}


sub get_upload_progress {
	print $LOG "get_upload_progress\n" if $DEBUG;
	
	
	##==============================
	#
	#$user = $Q->param('login');
	#$password = $Q->param('passwort');
	#print $LOG "> $ENV{login}\n";
	#
	##-- User Check
	#if ( ! &check_user($user, $password) ) {
	#	&welcome_screen;
	#}
    #
	##-- Check rights
	#if ( $USER_LEVEL < GUEST_UP ) {
	#	$_error = "No authorization.";
	#	&welcome_screen;
	#	exit;
	#}
    #
	#
	##if (! $filename) {
	##	$_error = "Bad Filename !";
	##	&welcome_screen;
	##}
    ##
	##
	##if (-e "$user_dir/$filename") {
	##	$_error = "Filename already exists, choose an other one !";
	##	&welcome_screen;
	##}
	#
	
	
	# ========================================
	
	
	
	
	
	
	my $pars = $ENV{QUERY_STRING};
	my ($ufile) = ($pars =~ /file=(\w+\/.+)$/);
	my ($user, $file) = split('\/', $ufile);
	#-- extract filename
	$_ = $file;
	s/\w://;
	s/([^\/\\]+)$//;
	$_ = $1;
	s/\.\.+//g;
	s/\s+//g;
	s/^.*\/(.+)$/$1/;		# (thanks OSX)
	#-- URI-Filler umcodieren
	s/(\%)([0-9a-fA-F]{2})/pack("H2", $2)/eg;
	$file = $_;
	
	#&read_user_data($user);
	
	open(TMP_AJAX,"${DATA_ROOT_TMP}/${file}_TMP_AJAX")or print $LOG "err: file:${DATA_ROOT_TMP}/${file}_TMP_AJAX :$!\n";
    my $in = <TMP_AJAX>;
    close TMP_AJAX;
	chomp($in);
	
	# $in ist: $upload_file_size|$bytes_read
	
	print $LOG "FILE=${file} / IN=$in\n" if $DEBUG;
	
	print "Cache-Control: no-store, no-cache\n";
	print "Content-type: text/html\n\n";
	
	$in ? print $in : print "UNDEF|UNDEF";
	
	exit;
}





#-- ADMIN

sub admin_start {
	print $LOG "=== admin_start\n";
	print $LOG $Q->param(), "\n";
	
	print $Q->header();
	print $Q->start_html( -title => "Admin",
			-meta => { 	cache_control => 'no-cache',
							expires => 'now',
							pragma => 'no-cache',
							robots => 'noindex' },
			-author => "$AUTHORMAIL",
			-alink => 'red',
			-link => 'ivory',
			-vlink => 'ivory',
			-style => {-code => "$STYLE_1"},
			-BGCOLOR => "$BGCOLOR" );
	
	print qq(<center><p>$LOGO_ADMIN</center>\n);
			
	print $Q->start_multipart_form(	-method => 'POST', -name => 'startform');
	
	#-- Login
	print qq(<hr size=1>);
	print qq(<center><p>);

	print qq(<table width=50% bgcolor=$LISTBG><tr><td>&nbsp</td></tr>);
	print qq(<tr><td align="left"> User : </td><td><input type="text" name="a_login"  size=15></td></tr>);
	print qq(<tr><td align="left"> Passwort : </td><td><input type="password" name="a_passwort"  size=15></td>);
	print qq(<br><td align="right"> </td><td>);
	print $Q->submit(	-name => "a_start", -value => "Log In");
	print qq(</td></tr>);
	print qq(<tr><td>&nbsp</td></tr></table>);

	print "<br><center><font color=red><b>" . $_error . "</b></font></center>" if $_error;

	print $Q->endform();

	goto end_script;
}


sub admin_screen {
	print $LOG "=== admin_screen\n";
	print $LOG $Q->param(), "\n";

	my $a_login = $Q->param('a_login');
	my $a_passwort = $Q->param('a_passwort');
	my $save_error = $_error;
		
	if ( ! &check_user( $a_login, $a_passwort ) ) {
		$_error = "User / Password not valid !";
		&admin_start;
	}
	if ( $USER_LEVEL != ADMIN ) {
		$_error = "Not a valid Administrator !";
		&admin_start;
	}
	
	$_error = $save_error if not $_error;
				
	print $Q->header();
	print $Q->start_html( -title => "Admin",
			-meta => { 	cache_control => 'no-cache',
							expires => 'now',
							pragma => 'no-cache',
							robots => 'noindex' },
			-author => "$AUTHORMAIL",
			-alink => 'red',
			-link => 'ivory',
			-vlink => 'ivory',
			-style => {-code => "$STYLE_1"},
			-BGCOLOR => "$BGCOLOR" );
	
	print qq(<center><p>$LOGO_ADMIN</center>\n);
	print qq(<hr size=1><br>);
			
	print $Q->start_multipart_form(	-method => 'POST', -name => 'adminform');
	
	
	print qq(<center><table width=80% bgcolor=$LISTBG border><tr><th> </th><th>Login</th><th>Level</th><th>User Data Path</th><th>Web Location Root</th></tr>);

	my @data = &read_data_file;
	foreach my $line ( sort { substr($a,0,5) cmp substr($b,0,5) } @data ) {
		next if $line !~ /^\w+\|/;
		my( $user, $pw, $level, $data_root, $web_location) = split('\|', $line);
		print qq(<tr>);
		print qq(<td align="center"><input type=radio name='radio' value="$user"></td>);
		print qq(<td>$user</td><td>$LEV{"$level"}</td><td>$data_root</td><td>$web_location</td></tr>);
	
	}
	print qq(</table>);

	print "<br><center><font color=red><b>" . $_error . "</b></font></center>" if $_error;
	
	print qq(<p><center><table width=80%><tr><td>);
	print $Q->submit( -name => "adm_edit", -value => "Edit");
	print qq(&nbsp&nbsp&nbsp&nbsp);
	print $Q->submit( -name => "adm_add", -value => "Add");
	print qq(&nbsp&nbsp&nbsp&nbsp);
	print $Q->submit( -name => "adm_delete", -value => "Delete");
	print qq(&nbsp&nbsp&nbsp);
	print $Q->checkbox( -name => "adm_del_dir", -value => "adm_del_dir", -label => "Remove Directories too !");
	print qq(</td></tr></table>);
	
	print $Q->hidden( -name => "a_login", -default => "$a_login" );
	print $Q->hidden( -name => "a_passwort", -default => "$a_passwort" );
	
	print $Q->endform();
	

	print $FOOTER;
	
	goto end_script;	
}



sub admin_edit {
	print $LOG "=== admin_edit\n";
	print $LOG $Q->param(), "\n";
	
	my $login = $Q->param('radio');
	
	print $Q->header();
	print $Q->start_html( -title => "Admin",
			-meta => { 	cache_control => 'no-cache',
							expires => 'now',
							pragma => 'no-cache',
							robots => 'noindex' },
			-author => "$AUTHORMAIL",
			-alink => 'red',
			-link => 'ivory',
			-vlink => 'ivory',
			-style => {-code => "$STYLE_1"},
			-BGCOLOR => "$BGCOLOR" );
	
	print qq(<center><p>$LOGO_ADMIN</center>\n);
	print qq(<hr size=1><br>);
	
	&read_user_data( $login );
	
	my @vals = keys(%LEV);
			
	print $Q->start_multipart_form(	-method => 'POST', -name => 'adminform');
	
	print qq(<center><table>);
	print qq(<tr><td align="left"> Login (Gruppenname) : </td><td><b>$login</b></td></tr>);
	print qq(<tr><td align="left"> <br> </td></tr>);
	print qq(<tr><td align="left"> Passwort : </td><td><input type="password" name="passwort"  size=10></td></tr>);
	print qq(<tr><td align="left"> Level : </td><td>);
	print $Q->popup_menu( -name => "level", -values => \@vals, -labels => \%LEV, -default => "$LEVEL" );
	print qq(</td></tr>);
	print qq(<tr><td align="left"> User Data Path : </td><td><input type="text" name="user_data_path" value="$USER_DATA_ROOT" size=40 maxlength=120></td></tr>);
	print qq(<tr><td align="left"> Web Location : </td><td><input type="text" name="web_location" value="$WEB_LOCATION" size=40 maxlength=120 readonly></td></tr>);
	print qq(<tr><td align="left"> <br> </td></tr>);
	print qq(<tr><td align="left">);
	print $Q->submit( -name => "adm_modifyit", -value => "Modify It");
	print $Q->hidden( "login", "$login" );
	
	print qq(</td></tr></table></center>);
	
	print $Q->hidden( -name => "a_login", -default => $Q->param('a_login') );
	print $Q->hidden( -name => "a_passwort", -default => $Q->param('a_passwort') );
	
	print $Q->endform();
	
	print $FOOTER_JS_B;
	
	goto end_script;	
	
}


sub admin_add {
	print $LOG "=== admin_add\n";
	print $LOG $Q->param(), "\n";
	
	
	print $Q->header();
	print $Q->start_html( -title => "Admin",
			-meta => { 	cache_control => 'no-cache',
							expires => 'now',
							pragma => 'no-cache',
							robots => 'noindex' },
			-author => "$AUTHORMAIL",
			-alink => 'red',
			-link => 'ivory',
			-vlink => 'ivory',
			-style => {-code => "$STYLE_1"},
			-BGCOLOR => "$BGCOLOR" );
	
	print qq(<center><p>$LOGO_ADMIN</center>\n);
	print qq(<hr size=1><br>);
	
	
	
	my @vals = keys(%LEV);
			
	print $Q->start_multipart_form(	-method => 'POST', -name => 'adminform');
	
	print qq(<center><table>);
	print qq(<tr><td align="left"> Login (Gruppenname) : </td><td><input type="test" name="login"  size=10></td></tr>);
	print qq(<tr><td align="left"> <br> </td></tr>);
	print qq(<tr><td align="left"> Passwort : </td><td><input type="password" name="passwort"  size=10></td></tr>);
	print qq(<tr><td align="left"> Level : </td><td>);
	print $Q->popup_menu( -name => "level", -values => \@vals, -labels => \%LEV, -default => GUEST );
	print qq(</td></tr>);
	print qq(<tr><td align="left"> User Data Path : </td><td><input type="text" name="user_data_path" value="$DATA_ROOT" size=40 maxlength=120></td></tr>);
	print qq(<tr><td align="left"> Web Location : </td><td><input type="text" name="web_location" value="$WEB_LOCATION" size=40 maxlength=120 readonly></td></tr>);
	print qq(<tr><td align="left"> <br> </td></tr>);
	print qq(<tr><td align="left">);
	print $Q->submit( -name => "adm_addit", -value => "Add It");
	
	print qq(</td></tr></table></center>);
	
	print $Q->hidden( -name => "a_login", -default => $Q->param('a_login') );
	print $Q->hidden( -name => "a_passwort", -default => $Q->param('a_passwort') );
	
	print $Q->endform();
	
	print $FOOTER_JS_B;
	
	goto end_script;	
	
}



sub admin_modifyit {
	print $LOG "=== admin_modifyit\n";
	print $LOG $Q->param(), "\n";

	my $login 			= $Q->param('login');
	my $passwort 		= $Q->param('passwort');
	chomp($passwort);
	my $level 			= $Q->param('level');
	my $user_data_path 	= $Q->param('user_data_path');
	my $web_location 	= $Q->param('web_location');
	
	if ( $passwort eq "" ) {
		$_error = "Leeres Passwort nicht erlaubt.";
		&admin_screen;
		exit;
	}
	
	&read_user_data( $login );
	
	$MOD_PATH = TRUE if noslash($user_data_path) ne noslash($USER_DATA_ROOT);
	
	$passwort = $USER_PW if $passwort eq "";
			
	if ( &write_data ( $login, $passwort, $level, $user_data_path, $web_location ) ) {
		&admin_screen;
		exit;
	} else { print $LOG "ERR:$_error\n";
		&admin_screen;
		exit;
	}		
	
	die "oops: $!\n";
	
}
	
sub admin_addit {
	print $LOG "=== admin_addit\n";
	print $LOG $Q->param(), "\n";

	my $login 			= $Q->param('login');
	my $passwort 		= $Q->param('passwort');
	my $level 			= $Q->param('level');
	my $user_data_path 	= $Q->param('user_data_path');
	my $web_location 	= $Q->param('web_location');
	
	if ( $passwort eq "" ) {
		$_error = "Leeres Passwort nicht erlaubt.";
		&admin_screen;
		exit;
	}
	
	print $LOG "IN: $user_data_path / DR: $DATA_ROOT\n";
	if ( noslash($user_data_path) eq noslash($DATA_ROOT) ) {
		$_error = "User Data Path not a valid path ! No action taken.";
		&admin_screen;
		exit;
	}
			
	if ( &write_data ( $login, $passwort, $level, $user_data_path, $web_location ) ) {
		&admin_screen;
		exit;
	} else {
		&admin_screen;
		exit;
	}		
	
	die "oops: $!\n";
	
}


sub adm_delete {
	print $LOG "=== adm_delete\n";
	print $LOG $Q->param(), "\n";

	my $login = $Q->param('radio');
	my $a_login = $Q->param('a_login');
	
	if ( $a_login eq $login ) {
		$_error = "I do not want to delete myself !";
		&admin_screen;
		exit;
	}
	
	&read_user_data( $login ) if $login;
	
	if ( $Q->param('adm_del_dir') eq "adm_del_dir" && $login ne "" ) {
		print $LOG "rmdir: $USER_DATA_ROOT\n";
		rmtree( $USER_DATA_ROOT ) if noslash($USER_DATA_ROOT) ne noslash($DATA_ROOT);
		print $LOG "rmdir: $DOCUMENT_ROOT/$WEB_LOCATION/$login\n";
		rmtree( "$DOCUMENT_ROOT/$WEB_LOCATION/$login" ) if noslash("$DOCUMENT_ROOT/$WEB_LOCATION/$login") ne noslash("$DOCUMENT_ROOT/$WEB_LOCATION/");
	}
		
    if ( ! open(PWD_OUT, ">$PWDTEMP") ) {
    	$_error = "Cannot open $PWDTEMP: $!";
    	return FALSE;
    }		
	if ( ! open(PWD, "$PWDFILE") ) {
		$_error = "Cannot open $PWDFILE: $!";	
		return FALSE;
	}
	
	my $found = FALSE;
	while ( my $line = <PWD> ) {
	    next if $line =~ /^$login\|/;
	    print PWD_OUT $line;
	}
	close PWD;
	close PWD_OUT;

	system("mv $PWDTEMP $PWDFILE");
	
	&admin_screen;
	exit;
}

